@@ -41,6 +41,7 @@
 #endif
 #include <amqp.h>
 #include <amqp_framing.h>
+#include <amqp_tcp_socket.h>
 
 #ifdef PHP_WIN32
 # include "win32/unistd.h"
@@ -54,6 +55,9 @@
 #define E_DEPRECATED E_WARNING
 #endif
 
+int php_amqp_set_read_timeout(amqp_connection_object *connection TSRMLS_DC);
+int php_amqp_set_write_timeout(amqp_connection_object *connection TSRMLS_DC);
+
 #if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3
 zend_object_handlers amqp_connection_object_handlers;
 HashTable *amqp_connection_object_get_debug_info(zval *object, int *is_temp TSRMLS_DC) {
@@ -100,6 +104,14 @@ HashTable *amqp_connection_object_get_debug_info(zval *object, int *is_temp TSRM
 	ZVAL_DOUBLE(value, connection->write_timeout);
 	zend_hash_add(debug_info, "write_timeout", sizeof("write_timeout"), &value, sizeof(zval *), NULL);
 
+	MAKE_STD_ZVAL(value);
+	ZVAL_DOUBLE(value, connection->connect_timeout);
+	zend_hash_add(debug_info, "connect_timeout", sizeof("connect_timeout"), &value, sizeof(zval *), NULL);
+
+	MAKE_STD_ZVAL(value);
+	ZVAL_BOOL(value, connection->is_connected);
+	zend_hash_add(debug_info, "is_connected", sizeof("is_connected"), &value, sizeof(zval *), NULL);
+
 	/* Start adding values */
 	return debug_info;
 }
@@ -118,7 +130,8 @@ int php_amqp_connect(amqp_connection_object *connection, int persistent TSRMLS_D
 	void * old_handler;
 #endif
 	amqp_rpc_reply_t x;
-
+	struct timeval tv = {0};
+	struct timeval *tv_ptr = &tv;
 
 	/* Clean up old memory allocations which are now invalid (new connection) */
 	if (connection->connection_resource) {
@@ -157,11 +170,18 @@ int php_amqp_connect(amqp_connection_object *connection, int persistent TSRMLS_D
 	/* Create the connection */
 	connection->connection_resource->connection_state = amqp_new_connection();
 
-	/* Get the connection socket out */
-	connection->connection_resource->fd = amqp_open_socket(connection->host, connection->port);
+	/* Create socket object */
+	connection->connection_resource->socket = amqp_tcp_socket_new(connection->connection_resource->connection_state);
 
-	/* Verify that we actually got a connectio back */
-	if (connection->connection_resource->fd < 1) {
+	if (connection->connect_timeout > 0) {
+		tv.tv_sec = (long int) connection->connect_timeout;
+		tv.tv_usec = (long int) ((connection->connect_timeout - tv.tv_sec) * 1000000);
+	} else {
+		tv_ptr = NULL;
+	}
+
+	/* Try to connect and verify that no error occurred */
+	if (amqp_socket_open_noblock(connection->connection_resource->socket, connection->host, connection->port, tv_ptr)) {
 #ifndef PHP_WIN32
 		/* Start ignoring SIGPIPE */
 		old_handler = signal(SIGPIPE, SIG_IGN);
@@ -178,16 +198,14 @@ int php_amqp_connect(amqp_connection_object *connection, int persistent TSRMLS_D
 		return 0;
 	}
 
-	amqp_set_sockfd(connection->connection_resource->connection_state, connection->connection_resource->fd);
-
 	php_amqp_set_read_timeout(connection TSRMLS_CC);
 	php_amqp_set_write_timeout(connection TSRMLS_CC);
 
 	x = amqp_login(
 		connection->connection_resource->connection_state,
 		connection->vhost,
-		CHANNEL_MAX,
-		FRAME_MAX,
+		AMQP_DEFAULT_MAX_CHANNELS,
+		AMQP_DEFAULT_FRAME_SIZE,
 		AMQP_HEARTBEAT,
 		AMQP_SASL_METHOD_PLAIN,
 		connection->login,
@@ -195,7 +213,8 @@ int php_amqp_connect(amqp_connection_object *connection, int persistent TSRMLS_D
 	);
 
 	if (x.reply_type != AMQP_RESPONSE_NORMAL) {
-		amqp_error(x, pstr);
+		amqp_error(x, pstr, connection, NULL);
+
 		strcat(*pstr, " - Potential login failure.");
 		zend_throw_exception(amqp_connection_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		return 0;
@@ -216,6 +235,7 @@ void php_amqp_disconnect(amqp_connection_object *connection)
 	void * old_handler;
 #endif
 	int slot;
+	amqp_channel_object *channel;
 
 	/* Pull the connection resource out for easy access */
 	amqp_connection_resource *resource = connection->connection_resource;
@@ -242,6 +262,9 @@ void php_amqp_disconnect(amqp_connection_object *connection)
 				if ((long) resource->slots[slot] != -1) {
 					/* We found the channel, disconnect it: */
 					amqp_channel_close(connection->connection_resource->connection_state, slot, AMQP_REPLY_SUCCESS);
+
+					channel = (amqp_channel_object *) resource->slots[slot];
+					channel->is_connected = '\0';
 				}
 
 				/* Clean up our local storage */
@@ -255,15 +278,10 @@ void php_amqp_disconnect(amqp_connection_object *connection)
 	if (resource && resource->connection_state && connection->is_connected == '\1') {
 		amqp_connection_close(resource->connection_state, AMQP_REPLY_SUCCESS);
 		amqp_destroy_connection(resource->connection_state);
-
-		if (resource->fd) {
-			AMQP_CLOSE_SOCKET(resource->fd)
-		}
 	}
 
 	connection->is_connected = '\0';
 
-
 #ifndef PHP_WIN32
 	/* End ignoring of SIGPIPEs */
 	signal(SIGPIPE, old_handler);
@@ -293,7 +311,7 @@ int php_amqp_set_read_timeout(amqp_connection_object *connection TSRMLS_DC)
 	read_timeout.tv_usec = (int) ((connection->read_timeout - floor(connection->read_timeout)) * 1.e+6);
 #endif
 
-	if (0 != setsockopt(connection->connection_resource->fd, SOL_SOCKET, SO_RCVTIMEO, (char *)&read_timeout, sizeof(read_timeout))) {
+	if (0 != setsockopt(amqp_get_sockfd(connection->connection_resource->connection_state), SOL_SOCKET, SO_RCVTIMEO, (char *)&read_timeout, sizeof(read_timeout))) {
 		zend_throw_exception(amqp_connection_exception_class_entry, "Socket error: cannot setsockopt SO_RCVTIMEO", 0 TSRMLS_CC);
 		return 0;
 	}
@@ -317,7 +335,7 @@ int php_amqp_set_write_timeout(amqp_connection_object *connection TSRMLS_DC)
 	write_timeout.tv_usec = (int) ((connection->write_timeout - floor(connection->write_timeout)) * 1.e+6);
 #endif
 
-	if (0 != setsockopt(connection->connection_resource->fd, SOL_SOCKET, SO_SNDTIMEO, (char *)&write_timeout, sizeof(write_timeout))) {
+	if (0 != setsockopt(amqp_get_sockfd(connection->connection_resource->connection_state), SOL_SOCKET, SO_SNDTIMEO, (char *)&write_timeout, sizeof(write_timeout))) {
 		zend_throw_exception(amqp_connection_exception_class_entry, "Socket error: cannot setsockopt SO_SNDTIMEO", 0 TSRMLS_CC);
 		return 0;
 	}
@@ -355,9 +373,12 @@ int get_next_available_channel(amqp_connection_object *connection, amqp_channel_
 void remove_channel_from_connection(amqp_connection_object *connection, amqp_channel_object *channel)
 {
 	int slot;
+	amqp_connection_resource *resource;
+
+	channel->is_connected = '\0';
 
 	/* Pull out the ring buffer for ease of use */
-	amqp_connection_resource *resource = connection->connection_resource;
+	resource = connection->connection_resource;
 
 	/* Check that there is actually an open connection */
 	if (!resource) {
@@ -469,7 +490,7 @@ zend_object_value amqp_connection_ctor(zend_class_entry *ce TSRMLS_DC)
 
 
 /* {{{ proto AMQPConnection::__construct([array optional])
- * The array can contain 'host', 'port', 'login', 'password', 'vhost', 'read_timeout', 'write_timeout' and 'timeout' (deprecated) indexes
+ * The array can contain 'host', 'port', 'login', 'password', 'vhost', 'read_timeout', 'write_timeout', 'connect_timeout' and 'timeout' (deprecated) indexes
  */
 PHP_METHOD(amqp_connection_class, __construct)
 {
@@ -614,6 +635,17 @@ PHP_METHOD(amqp_connection_class, __construct)
 			connection->write_timeout = Z_DVAL_PP(zdata);
 		}
 	}
+
+	connection->connect_timeout = INI_FLT("amqp.connect_timeout");
+
+	if (ini_arr && SUCCESS == zend_hash_find(HASH_OF (ini_arr), "connect_timeout", sizeof("connect_timeout"), (void*)&zdata)) {
+		convert_to_double(*zdata);
+		if (Z_DVAL_PP(zdata) < 0) {
+			zend_throw_exception(amqp_connection_exception_class_entry, "Parameter 'connect_timeout' must be greater than or equal to zero.", 0 TSRMLS_CC);
+		} else {
+			connection->connect_timeout = Z_DVAL_PP(zdata);
+		}
+	}
 }
 /* }}} */
 
@@ -860,6 +892,11 @@ PHP_METHOD(amqp_connection_class, setLogin)
 	/* Get the connection object out of the store */
 	connection = (amqp_connection_object *)zend_object_store_get_object(id TSRMLS_CC);
 
+	/* Free previously existing login, in cases where setLogin() is called multiple times */
+	if (connection->login) {
+		efree(connection->login);
+	}
+
 	/* Copy the login to the amqp object */
 	connection->login = estrndup(login, login_len);
 
@@ -911,6 +948,11 @@ PHP_METHOD(amqp_connection_class, setPassword)
 	/* Get the connection object out of the store */
 	connection = (amqp_connection_object *)zend_object_store_get_object(id TSRMLS_CC);
 
+	/* Free previously existing password, in cases where setPassword() is called multiple times */
+	if (connection->password) {
+		efree(connection->password);
+	}
+
 	/* Copy the password to the amqp object */
 	connection->password = estrndup(password, password_len);
 
@@ -963,6 +1005,11 @@ PHP_METHOD(amqp_connection_class, setHost)
 	/* Get the connection object out of the store */
 	connection = (amqp_connection_object *)zend_object_store_get_object(id TSRMLS_CC);
 
+	/* Free previously existing host, in cases where setHost() is called multiple times */
+	if (connection->host) {
+		efree(connection->host);
+	}
+
 	/* Copy the host to the amqp object */
 	connection->host = estrndup(host, host_len);
 
@@ -1082,6 +1129,11 @@ PHP_METHOD(amqp_connection_class, setVhost)
 	/* Get the connection object out of the store */
 	connection = (amqp_connection_object *)zend_object_store_get_object(id TSRMLS_CC);
 
+	/* Free previously existing vhost, in cases where setVhost() is called multiple times */
+	if (connection->vhost) {
+		efree(connection->vhost);
+	}
+
 	/* Copy the vhost to the amqp object */
 	connection->vhost = estrndup(vhost, vhost_len);
 
