@@ -83,6 +83,10 @@ HashTable *amqp_exchange_object_get_debug_info(zval *object, int *is_temp TSRMLS
 	MAKE_STD_ZVAL(value);
 	ZVAL_LONG(value, exchange->durable);
 	zend_hash_add(debug_info, "durable", sizeof("durable"), &value, sizeof(zval *), NULL);
+	
+	MAKE_STD_ZVAL(value);
+	ZVAL_LONG(value, exchange->auto_delete);
+	zend_hash_add(debug_info, "auto_delete", sizeof("auto_delete"), &value, sizeof(zval *), NULL);
 
 	zend_hash_add(debug_info, "arguments", sizeof("arguments"), &exchange->arguments, sizeof(&exchange->arguments), NULL);
 
@@ -175,7 +179,7 @@ PHP_METHOD(amqp_exchange_class, __construct)
 	amqp_channel_object *channel;
 
 	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "OO", &id, amqp_exchange_class_entry, &channelObj, amqp_channel_class_entry) == FAILURE) {
-		zend_throw_exception(amqp_exchange_exception_class_entry, "Parameter must be an instance of AMQPConnection.", 0 TSRMLS_CC);
+		zend_throw_exception(amqp_exchange_exception_class_entry, "Parameter must be an instance of AMQPChannel.", 0 TSRMLS_CC);
 		RETURN_NULL();
 	}
 
@@ -242,8 +246,8 @@ PHP_METHOD(amqp_exchange_class, setName)
 	exchange = (amqp_exchange_object *)zend_object_store_get_object(id TSRMLS_CC);
 
 	/* Verify that the name is not null and not an empty string */
-	if (name_len < 1 || name_len > 255) {
-		zend_throw_exception(amqp_exchange_exception_class_entry, "Invalid exchange name given, must be between 1 and 255 characters long.", 0 TSRMLS_CC);
+	if (name_len > 255) {
+		zend_throw_exception(amqp_exchange_exception_class_entry, "Invalid exchange name given, must be less than 255 characters long.", 0 TSRMLS_CC);
 		return;
 	}
 
@@ -270,6 +274,7 @@ PHP_METHOD(amqp_exchange_class, getFlags)
 	/* Set the bitmask based on what is set in the exchange */
 	flagBitmask |= (exchange->passive ? AMQP_PASSIVE : 0);
 	flagBitmask |= (exchange->durable ? AMQP_DURABLE : 0);
+	flagBitmask |= (exchange->auto_delete ? AMQP_AUTODELETE : 0);
 
 	RETURN_LONG(flagBitmask);
 }
@@ -291,12 +296,10 @@ PHP_METHOD(amqp_exchange_class, setFlags)
 	/* Pull the exchange off the object store */
 	exchange = (amqp_exchange_object *)zend_object_store_get_object(id TSRMLS_CC);
 
-
-
 	/* Set the flags based on the bitmask we were given */
 	exchange->passive = IS_PASSIVE(flagBitmask);
 	exchange->durable = IS_DURABLE(flagBitmask);
-    exchange->auto_delete = IS_AUTODELETE(flagBitmask);
+	exchange->auto_delete = IS_AUTODELETE(flagBitmask);
 }
 /* }}} */
 
@@ -339,7 +342,8 @@ PHP_METHOD(amqp_exchange_class, setType)
 
 	/* Pull the exchange off the object store */
 	exchange = (amqp_exchange_object *)zend_object_store_get_object(id TSRMLS_CC);
-	AMQP_SET_TYPE(exchange, type)
+	
+	AMQP_SET_TYPE(exchange, type);
 }
 /* }}} */
 
@@ -366,6 +370,7 @@ PHP_METHOD(amqp_exchange_class, getArgument)
 	}
 
 	*return_value = **tmp;
+	
 	zval_copy_ctor(return_value);
 	INIT_PZVAL(return_value);
 }
@@ -498,6 +503,7 @@ PHP_METHOD(amqp_exchange_class, declareExchange)
 	}
 
 	arguments = convert_zval_to_arguments(exchange->arguments);
+	
 	amqp_exchange_declare(
 		connection->connection_resource->connection_state,
 		channel->channel_id,
@@ -516,7 +522,8 @@ PHP_METHOD(amqp_exchange_class, declareExchange)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char ** pstr = (char **) &str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		zend_throw_exception(amqp_exchange_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		efree(*pstr);
 		return;
@@ -583,7 +590,8 @@ PHP_METHOD(amqp_exchange_class, delete)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char ** pstr = (char **) &str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		zend_throw_exception(amqp_exchange_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
 		return;
@@ -785,7 +793,7 @@ PHP_METHOD(amqp_exchange_class, publish)
 			} else if (Z_TYPE_P(*zdata) == IS_DOUBLE) {
 				convert_to_double(*zdata);
 				props.headers.entries[props.headers.num_entries].value.kind = AMQP_FIELD_KIND_F32;
-				props.headers.entries[props.headers.num_entries].value.value.f32 = (double)Z_DVAL_P(*zdata);
+				props.headers.entries[props.headers.num_entries].value.value.f32 = (float)Z_DVAL_P(*zdata);
 				props.headers.num_entries++;
 			} else if (Z_TYPE_PP(zdata) == IS_ARRAY) {
 				zval **arr_data;
@@ -865,7 +873,7 @@ PHP_METHOD(amqp_exchange_class, publish)
 	if (r < 0) {
 		char str[256];
 		char ** pstr = (char **) &str;
-        spprintf(pstr, 0, "Socket error: %s", amqp_error_string(-r));
+        spprintf(pstr, 0, "Socket error: %s", amqp_error_string2(-r));
         zend_throw_exception(amqp_exchange_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		return;
 	}
@@ -934,7 +942,8 @@ PHP_METHOD(amqp_exchange_class, bind)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char ** pstr = (char **) &str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		zend_throw_exception(amqp_exchange_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
 		return;
@@ -945,6 +954,41 @@ PHP_METHOD(amqp_exchange_class, bind)
 }
 /* }}} */
 
+/* {{{ proto AMQPExchange::getChannel()
+Get the AMQPChannel object in use */
+PHP_METHOD(amqp_exchange_class, getChannel)
+{
+	zval *id;
+	amqp_exchange_object *exchange;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O", &id, amqp_exchange_class_entry) == FAILURE) {
+		return;
+	}
+
+	exchange = (amqp_exchange_object *)zend_object_store_get_object(id TSRMLS_CC);
+
+    RETURN_ZVAL(exchange->channel, 1, 0);
+}
+/* }}} */
+
+/* {{{ proto AMQPExchange::getConnection()
+Get the AMQPConnection object in use */
+PHP_METHOD(amqp_exchange_class, getConnection)
+{
+	zval *id;
+	amqp_exchange_object *exchange;
+	amqp_channel_object *channel;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O", &id, amqp_exchange_class_entry) == FAILURE) {
+		return;
+	}
+
+	exchange = (amqp_exchange_object *)zend_object_store_get_object(id TSRMLS_CC);
+	channel = AMQP_GET_CHANNEL(exchange);
+
+    RETURN_ZVAL(channel->connection, 1, 0);
+}
+/* }}} */
 
 /*
 *Local variables:
