@@ -274,7 +274,7 @@ int read_message_from_channel(amqp_connection_state_t connection, zval *envelope
 
 		/* Check that the basic read from frame did not fail */
 		if (result < 0) {
-			zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string(-result), -result TSRMLS_CC);
+			zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string2(-result), -result TSRMLS_CC);
 			return AMQP_READ_ERROR;
 		}
 
@@ -331,7 +331,7 @@ int read_message_from_channel(amqp_connection_state_t connection, zval *envelope
 		/* Read in the next frame */
 		result = amqp_simple_wait_frame(connection, &frame);
 		if (result < 0) {
-			zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string(-result), -result TSRMLS_CC);
+			zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string2(-result), -result TSRMLS_CC);
 			return AMQP_READ_ERROR;
 		}
 
@@ -415,7 +415,7 @@ int read_message_from_channel(amqp_connection_state_t connection, zval *envelope
 			/* Read in the next frame */
 			result = amqp_simple_wait_frame(connection, &frame);
 			if (result < 0) {
-				zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string(-result), -result TSRMLS_CC);
+				zend_throw_exception(amqp_connection_exception_class_entry, amqp_error_string2(-result), -result TSRMLS_CC);
 				return AMQP_READ_ERROR;
 			}
 
@@ -638,10 +638,10 @@ PHP_METHOD(amqp_queue_class, getArguments)
 Overwrite all queue arguments with given args */
 PHP_METHOD(amqp_queue_class, setArguments)
 {
-	zval *id, *zvalArguments;
+	zval *id, *arguments;
 	amqp_queue_object *queue;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oa", &id, amqp_queue_class_entry, &zvalArguments) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oa", &id, amqp_queue_class_entry, &arguments) == FAILURE) {
 		return;
 	}
 
@@ -653,7 +653,7 @@ PHP_METHOD(amqp_queue_class, setArguments)
 		zval_ptr_dtor(&queue->arguments);
 	}
 
-	queue->arguments = zvalArguments;
+	queue->arguments = arguments;
 
 	/* Increment the ref count */
 	Z_ADDREF_P(queue->arguments);
@@ -757,7 +757,8 @@ PHP_METHOD(amqp_queue_class, declareQueue)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char ** pstr = (char **) &str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		channel->is_connected = '\0';
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
@@ -777,12 +778,12 @@ PHP_METHOD(amqp_queue_class, declareQueue)
 /* }}} */
 
 
-/* {{{ proto int AMQPQueue::bind(string exchangeName, [string routingKey]);
+/* {{{ proto int AMQPQueue::bind(string exchangeName, [string routingKey, array arguments]);
 bind queue to exchange by routing key
 */
 PHP_METHOD(amqp_queue_class, bind)
 {
-	zval *id;
+	zval *id, *arguments = NULL;
 	amqp_queue_object *queue;
 	amqp_channel_object *channel;
 	amqp_connection_object *connection;
@@ -795,8 +796,7 @@ PHP_METHOD(amqp_queue_class, bind)
 	amqp_queue_bind_t s;
 	amqp_method_number_t bind_ok = AMQP_QUEUE_BIND_OK_METHOD;
 
-
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|s", &id, amqp_queue_class_entry, &exchange_name, &exchange_name_len, &keyname, &keyname_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|sa", &id, amqp_queue_class_entry, &exchange_name, &exchange_name_len, &keyname, &keyname_len, &arguments) == FAILURE) {
 		return;
 	}
 
@@ -823,7 +823,11 @@ PHP_METHOD(amqp_queue_class, bind)
 	s.routing_key.bytes		= keyname;
 	s.nowait				= 0;
 	s.arguments.num_entries = 0;
-	s.arguments.entries	 	= NULL;
+	s.arguments.entries     = NULL;
+
+	if (arguments) {
+		s.arguments = *convert_zval_to_arguments(arguments);
+	}
 
 	res = AMQP_RPC_REPLY_T_CAST amqp_simple_rpc(
 		connection->connection_resource->connection_state,
@@ -836,7 +840,7 @@ PHP_METHOD(amqp_queue_class, bind)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char **pstr = (char **)&str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
 
 		channel->is_connected = 0;
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
@@ -907,7 +911,7 @@ PHP_METHOD(amqp_queue_class, get)
 	} else {
 		zval_ptr_dtor(&message);
 		RETURN_FALSE;
-    	}
+	}
 }
 /* }}} */
 
@@ -1236,7 +1240,8 @@ PHP_METHOD(amqp_queue_class, purge)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char **pstr = (char **)&str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		channel->is_connected = 0;
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
@@ -1307,7 +1312,8 @@ PHP_METHOD(amqp_queue_class, cancel)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char **pstr = (char **)&str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		channel->is_connected = 0;
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
@@ -1320,12 +1326,12 @@ PHP_METHOD(amqp_queue_class, cancel)
 /* }}} */
 
 
-/* {{{ proto int AMQPQueue::unbind(string exchangeName, [string routingKey]);
+/* {{{ proto int AMQPQueue::unbind(string exchangeName, [string routingKey, array arguments]);
 unbind queue from exchange
 */
 PHP_METHOD(amqp_queue_class, unbind)
 {
-	zval *id;
+	zval *id, *arguments = NULL;
 	amqp_queue_object *queue;
 	amqp_channel_object *channel;
 	amqp_connection_object *connection;
@@ -1339,7 +1345,7 @@ PHP_METHOD(amqp_queue_class, unbind)
 	amqp_queue_unbind_t s;
 	amqp_method_number_t method_ok = AMQP_QUEUE_UNBIND_OK_METHOD;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|s", &id, amqp_queue_class_entry, &exchange_name, &exchange_name_len, &keyname, &keyname_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|sa", &id, amqp_queue_class_entry, &exchange_name, &exchange_name_len, &keyname, &keyname_len, &arguments) == FAILURE) {
 		return;
 	}
 
@@ -1366,6 +1372,10 @@ PHP_METHOD(amqp_queue_class, unbind)
 	s.arguments.num_entries = 0;
 	s.arguments.entries		= NULL;
 
+	if (arguments) {
+		s.arguments = *convert_zval_to_arguments(arguments);
+	}
+
 	res = AMQP_RPC_REPLY_T_CAST amqp_simple_rpc(
 		connection->connection_resource->connection_state,
 		channel->channel_id,
@@ -1376,7 +1386,7 @@ PHP_METHOD(amqp_queue_class, unbind)
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char ** pstr = (char **) &str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
 
 		channel->is_connected = 0;
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
@@ -1391,7 +1401,7 @@ PHP_METHOD(amqp_queue_class, unbind)
 
 
 /* {{{ proto int AMQPQueue::delete([long flags = AMQP_NOPARAM]]);
-delete queue
+delete queue and return the number of messages deleted in it
 */
 PHP_METHOD(amqp_queue_class, delete)
 {
@@ -1402,10 +1412,10 @@ PHP_METHOD(amqp_queue_class, delete)
 
 	long flags = AMQP_NOPARAM;
 
+	amqp_queue_delete_ok_t *r;
 	amqp_rpc_reply_t res;
-	amqp_rpc_reply_t result;
-	amqp_queue_delete_t s;
-	amqp_method_number_t method_ok = AMQP_QUEUE_DELETE_OK_METHOD;
+
+	long message_count;
 
 	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|l", &id, amqp_queue_class_entry, &flags) == FAILURE) {
 		return;
@@ -1424,38 +1434,69 @@ PHP_METHOD(amqp_queue_class, delete)
 	connection = AMQP_GET_CONNECTION(channel);
 	AMQP_VERIFY_CONNECTION(connection, "Could not delete queue.");
 
-	s.queue.len		= queue->name_len;
-	s.queue.bytes	= queue->name;
-	s.ticket		= 0;
-	s.if_unused		= (AMQP_IFUNUSED & flags) ? 1 : 0;
-	s.if_empty		= (AMQP_IFEMPTY & flags) ? 1 : 0;
-	s.nowait		= 0;
-
-	result = amqp_simple_rpc(
+	r = amqp_queue_delete(
 		connection->connection_resource->connection_state,
 		channel->channel_id,
-		AMQP_QUEUE_DELETE_METHOD,
-		&method_ok,
-		&s
+		amqp_cstring_bytes(queue->name),
+		(AMQP_IFUNUSED & flags) ? 1 : 0,
+		(AMQP_IFEMPTY & flags) ? 1 : 0
 	);
 
-	res = AMQP_RPC_REPLY_T_CAST result;
+	res = AMQP_RPC_REPLY_T_CAST amqp_get_rpc_reply(connection->connection_resource->connection_state);
 
 	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
 		char str[256];
 		char **pstr = (char **)&str;
-		amqp_error(res, pstr);
+		amqp_error(res, pstr, connection, channel);
+
 		channel->is_connected = 0;
 		zend_throw_exception(amqp_queue_exception_class_entry, *pstr, 0 TSRMLS_CC);
 		amqp_maybe_release_buffers(connection->connection_resource->connection_state);
 		return;
 	}
+
+	message_count = r->message_count;
 	amqp_maybe_release_buffers(connection->connection_resource->connection_state);
 
-	RETURN_TRUE;
+	RETURN_LONG(message_count);
 }
 /* }}} */
 
+/* {{{ proto AMQPChannel::getChannel()
+Get the AMQPChannel object in use */
+PHP_METHOD(amqp_queue_class, getChannel)
+{
+	zval *id;
+	amqp_queue_object *queue;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|l", &id, amqp_queue_class_entry) == FAILURE) {
+		return;
+	}
+
+	queue = (amqp_queue_object *)zend_object_store_get_object(id TSRMLS_CC);
+
+    RETURN_ZVAL(queue->channel, 1, 0);
+}
+/* }}} */
+
+/* {{{ proto AMQPChannel::getConnection()
+Get the AMQPConnection object in use */
+PHP_METHOD(amqp_queue_class, getConnection)
+{
+	zval *id;
+	amqp_queue_object *queue;
+	amqp_channel_object *channel;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|l", &id, amqp_queue_class_entry) == FAILURE) {
+		return;
+	}
+
+	queue = (amqp_queue_object *)zend_object_store_get_object(id TSRMLS_CC);
+	channel = AMQP_GET_CHANNEL(queue);
+
+    RETURN_ZVAL(channel->connection, 1, 0);
+}
+/* }}} */
 
 /*
 *Local variables:
