@@ -149,7 +149,6 @@ PHP_MINIT_FUNCTION(amqp);
 PHP_MSHUTDOWN_FUNCTION(amqp);
 PHP_MINFO_FUNCTION(amqp);
 
-void amqp_error(amqp_rpc_reply_t x, char ** pstr);
 amqp_table_t *convert_zval_to_arguments(zval *zvalArguments);
 char *stringify_bytes(amqp_bytes_t bytes);
 
@@ -166,19 +165,14 @@ extern zend_class_entry *amqp_exception_class_entry,
 	*amqp_exchange_exception_class_entry,
 	*amqp_queue_exception_class_entry;
 
-
-#define FRAME_MAX							131072		/* max length (size) of frame */
-#define CHANNEL_MAX							10			/* max number of channels allowed */
-#define HEADER_FOOTER_SIZE					8			/*  7 bytes up front, then payload, then 1 byte footer */
 #define AMQP_HEARTBEAT						0	   		/* heartbeat */
 
-#define DEFAULT_NUM_CONNECTION_CHANNELS		255
-
 #define DEFAULT_PORT						"5672"		/* default AMQP port */
 #define DEFAULT_HOST						"localhost"
 #define DEFAULT_TIMEOUT						NULL
 #define DEFAULT_READ_TIMEOUT				"0"
 #define DEFAULT_WRITE_TIMEOUT				"0"
+#define DEFAULT_CONNECT_TIMEOUT			"0"
 #define DEFAULT_VHOST						"/"
 #define DEFAULT_LOGIN						"guest"
 #define DEFAULT_PASSWORD					"guest"
@@ -304,9 +298,9 @@ typedef struct _amqp_channel_object {
 typedef struct _amqp_connection_resource {
 	int used_slots;
 	amqp_channel_object **slots;
-	int fd;
 	int is_persistent;
 	amqp_connection_state_t connection_state;
+	amqp_socket_t *socket;
 } amqp_connection_resource;
 
 typedef struct _amqp_connection_object {
@@ -323,6 +317,7 @@ typedef struct _amqp_connection_object {
 	int port;
 	double read_timeout;
 	double write_timeout;
+	double connect_timeout;
 	amqp_connection_resource *connection_resource;
 } amqp_connection_object;
 
@@ -330,9 +325,9 @@ typedef struct _amqp_queue_object {
 	zend_object zo;
 	zval *channel;
 	char is_connected;
-	char name[255];
+	char name[256];
 	int name_len;
-	char consumer_tag[255];
+	char consumer_tag[256];
 	int consumer_tag_len;
 	int passive; /* @TODO: consider making these bit fields */
 	int durable;
@@ -346,9 +341,9 @@ typedef struct _amqp_exchange_object {
 	zend_object zo;
 	zval *channel;
 	char is_connected;
-	char name[255];
+	char name[256];
 	int name_len;
-	char type[255];
+	char type[256];
 	int type_len;
 	int passive; /* @TODO: consider making these bit fields */
 	int durable;
@@ -360,22 +355,22 @@ typedef struct _amqp_envelope_object {
 	zend_object zo;
 	char *body;
 	size_t body_len;
-	char routing_key[255];
+	char routing_key[256];
 	uint delivery_tag;
 	int delivery_mode;
-	char exchange_name[255];
+	char exchange_name[256];
 	int is_redelivery;
-	char content_type[255];
-	char content_encoding[255];
-	char type[255];
+	char content_type[256];
+	char content_encoding[256];
+	char type[256];
 	long timestamp;
 	int priority;
-	char expiration[255];
-	char user_id[255];
-	char app_id[255];
-	char message_id[255];
-	char reply_to[255];
-	char correlation_id[255];
+	char expiration[256];
+	char user_id[256];
+	char app_id[256];
+	char message_id[256];
+	char reply_to[256];
+	char correlation_id[256];
 	zval *headers;
 } amqp_envelope_object;
 
@@ -407,7 +402,9 @@ typedef struct _amqp_envelope_object {
 #define AMQP_G(v) (amqp_globals.v)
 #endif
 
-#define PHP_AMQP_VERSION "1.2.0"
+#define PHP_AMQP_VERSION "1.4.0"
+
+void amqp_error(amqp_rpc_reply_t x, char **pstr, amqp_connection_object *connection, amqp_channel_object *channel);
 
 #endif	/* PHP_AMQP_H */
 
